

#ifndef SOCKETTHREAD_HEADER
#define SOCKETTHREAD_HEADER

#include <stdlib.h>
#include <stdio.h>
#include <pthread.h>
#include <regex.h>
#include "dfsocket.h"

#ifdef __cplusplus
extern "C" {
#endif



#define DEBUG

#ifdef DEBUG
#define STDEBUG(...) { FILE *fd = fopen("/tmp/st.log","at"); fprintf(fd, "%s:%d %s() ", __FILE__, __LINE__, __func__); fprintf(fd, __VA_ARGS__); fprintf(fd, "\n"); fclose(fd); }
//#define STDEBUG(...) { FILE *fd = stdout; fprintf(fd, "%s:%d %s() ", __FILE__, __LINE__, __func__); fprintf(fd, __VA_ARGS__); fprintf(fd, "\n"); }
#else
#define STDEBUG(...)
#endif


typedef unsigned int st_thread;

struct __st_response {
	char *tag;
	
	char *value;
	unsigned int error;
	regex_t *regex;
	
	struct __st_response *next;
};
typedef struct __st_response st_response;

typedef struct {
	char *tag;
	
	char *send;
	
	regex_t *response_regex;
	int response_count;
	
	void (*callback)(st_response*, void*);
	void *context;
	
	unsigned int timeout;
} st_request;


typedef struct {
	char *description;
	unsigned int thread_stopping;
} st_error;


// setup the basic structures
int st_init(void);

// release all resources
int st_destroy(void);

// spawn a new socket thread
st_thread st_spawn(char *ipaddress, unsigned int port, regex_t *error_regex);

// release a socket thread
int st_release(st_thread id);

// request a less busy thread
st_thread st_requestthread(void);

// push a message to the threads queue
int st_send(st_thread id, st_request *request);

// register error callback
int st_register_errorcallback(st_thread id, void (*callback)(st_error*));
	
#ifdef __cplusplus
}
#endif

#endif